/*
   3D Barriers

   barrier.cc

   (c)2002 Paul Boese
*/

#include "common.h"
#include "barrier.h"

#define BARRIERS 9
#define BREAKING_FRAMES 60

enum { B_ACTIVE, B_HIT, B_BREAKING, B_GONE };

int Barrier::refcnt=0;

Barrier::Barrier() {
	mode = B_ACTIVE;
	id = refcnt++;
}

void Barrier::Draw(point_t * pos) {
	rots += 1.0f;
	switch (mode) {
	case B_ACTIVE:
		position.x = pos->x;
		position.y = pos->y;
		position.z = pos->z;
		draw_active();
		break;
	case B_HIT:
		snd_sfx_play(sfx_barrier_hit, gbl_sfx_volume, 128);
		mode = B_BREAKING;
	case B_BREAKING:
		mode = draw_breaking();
		break;
	case B_GONE:
		break;
	default:
		assert_msg(0, "Unknown Barrier mode");
	}
}

int Barrier::Active() {
	return !(mode == B_GONE)? -1 : 0;
}

void Barrier::Hit() {
	if (mode != B_GONE) {
		mode = B_HIT;
		pieces--;
		breaking = BREAKING_FRAMES;
	}
}

void Barrier::Reset(point_t *pos) {
	position.x = pos->x;
	position.y = pos->y;
	position.z = pos->z;
	mode = B_ACTIVE;
	pieces = 8;
	breaking = 0;
	rots = rotf=0.0f;
}

void Barrier::draw_active() {
	// draw barrier
	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotatef(rots, 0.0f, 0.5f, 0.5f);
	sphere(2.0f, 20, 20);
	glPopMatrix();
}

static float pulsate = 2.0f;
int Barrier::draw_breaking() {
	int rmode;
	
	// Draw barrier hit 
	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotatef(rots, 0.0f, 0.5f, 0.5f);
	sphere(pulsate, 20, 20);
	glPopMatrix();
	// Pulsate barrier size
	if (pieces > 0) {
		if (breaking-- > 0) {
			pulsate = 2.0f + ((float)breaking/(float)BREAKING_FRAMES)*0.3f*(fsin((rotf+=20.0f)*F_PI/180));
			rmode = B_BREAKING;
		} else {
			rotf = 0.0f;
			rmode =  B_ACTIVE;
		}
	} else {
		position.z = 100.0f;
		rmode = B_GONE;
	}
	return rmode;
}

/* Draws a sphere out of GL_QUADS according to the syntax of glutSolidSphere

   This now uses triangle strips and KGL/PVR Direct Rendering for the best
   possible performance.

   Based very loosely on a routine from James Susinno.

   Borrowed from Dan Potters' KGL bubble demo
*/
void Barrier::sphere( float radius, int slices, int stacks ) {
	int	i, j;
	float	pitch, pitch2;
	float	x, y, z, g, b;
	float	yaw, yaw2;
	pvr_vertex_t	*vert;
	pvr_dr_state_t	dr_state;

	/* Setup our Direct Render state: pick a store queue and setup QACR0/1 */
	pvr_dr_init(dr_state);

	/* Initialize xmtrx with the values from KGL */
	glKosMatrixIdent();
	glKosMatrixApply(GL_KOS_SCREENVIEW);
	glKosMatrixApply(GL_PROJECTION);
	glKosMatrixApply(GL_MODELVIEW);

	glKosPolyHdrSend();

	/* Iterate over stacks */
	for ( i=0; i<stacks; i++ ) {
		pitch = 2*M_PI * ( (float)i/(float)stacks );
		pitch2 = 2*M_PI * ( (float)(i+1)/(float)stacks );

		/* Iterate over slices: each entire stack will be one
		   long triangle strip. */
		for ( j=0; j<=slices/2; j++ ) {
			yaw = 2*M_PI * ( (float)j/(float)slices );
			yaw2 = 2*M_PI * ( (float)(j+1)/(float)slices );

			/* x, y+1 */
			vert = pvr_dr_target(dr_state);			/* Get a store queue address */
			vert->flags = PVR_CMD_VERTEX;
			x = radius * fcos( yaw ) * fcos( pitch2 );
			y = radius * fsin( pitch2 );
			z = radius * fsin( yaw ) * fcos( pitch2 );
			mat_trans_single(x, y, z);			/* Use ftrv to transform */
			vert->x = x;
			vert->y = y;
			vert->z = z;
			g = fcos( yaw*2 ) / 2.0f + 0.5f;
			b = fsin( pitch2 ) / 2.0f + 0.5f;
			vert->argb = 0x40000000
				| ((uint32)(g * 255) << 8)
				| ((uint32)(b * 255) << 0);
			pvr_dr_commit(vert);				/* Prefetch the SQ */
                
			/* x, y */
			vert = pvr_dr_target(dr_state);
			if (j == (slices/2))
				vert->flags = PVR_CMD_VERTEX_EOL;
			else
				vert->flags = PVR_CMD_VERTEX;
			x = radius * fcos( yaw ) * fcos( pitch );
			y = radius * fsin( pitch );
			z = radius * fsin( yaw ) * fcos( pitch );
			mat_trans_single(x, y, z);
			vert->x = x;
			vert->y = y;
			vert->z = z;
			g = fcos( yaw*2 ) / 2.0f + 0.5f;
			b = fsin( pitch ) / 2.0f + 0.5f;
			vert->argb = 0x40000000
				| ((uint32)(g * 255) << 8)
				| ((uint32)(b * 255) << 0);
			pvr_dr_commit(vert);
		}
	}
}


